@@ -12,7 +12,7 @@ import shutil
 import subprocess
 import sys
 
-from botocore.exceptions import ClientError
+from botocore.exceptions import ClientError, NoCredentialsError
 from urllib.parse import urlparse, parse_qs
 from time import sleep
 
@@ -115,6 +115,9 @@ class AwsSecretsManagerStore(DataStore):
             logging.error('Failed to retrieve secret {}; Error code: {}; Full error: {}'.format(
                 kwargs['SecretId'], ex.response['Error']['Code'], ex))
             raise RetrievalError
+        except NoCredentialsError:
+            logging.error('Could not retrieve secret because no AWS credentials were supplied')
+            raise RetrievalError
 
 
 class AwsSystemsManagerStore(DataStore):
@@ -157,6 +160,9 @@ class AwsSystemsManagerStore(DataStore):
             logging.error('Failed to retrieve parameter {}; Error code: {}; Full error: {}'.format(
                 kwargs['Name'], e.response['Error']['Code'], e))
             raise RetrievalError
+        except NoCredentialsError:
+            logging.error('Could not retrieve parameter because no AWS credentials were supplied')
+            raise RetrievalError
 
 
 class RedisStore(DataStore):
@@ -214,12 +220,14 @@ class LocalExecutionStore(DataStore):
     def get_value(self, key, **kwargs):
         try:
             subp_args = [ self.command ]
-            subp_args.extend(kwargs['arg'])
+            if 'arg' in kwargs:
+                subp_args.extend(kwargs['arg'])
 
             # Clean these args up a bit
             for i in range(0, len(subp_args)):
                 subp_args[i] = subp_args[i].strip()
 
+            logging.debug('Running command: {}'.format(' '.join(subp_args)))
             proc = subprocess.run(subp_args, capture_output=True)
             output = proc.stdout.decode('utf-8').strip()
             return output
@@ -250,7 +258,7 @@ class Retemplate(object):
         self.settings = {
             'owner': None,
             'group': None,
-            'chmod': 600,
+            'chmod': None,
             'onchange': None,
             'frequency': 60,
             'random_offset_max': None
@@ -272,6 +280,11 @@ class Retemplate(object):
 
         url = urlparse(uri)
         qs = parse_qs(url.query)
+        default = None
+        if 'rtpl_default' in qs:
+            default = ''.join(qs['rtpl_default'])
+            # The value must be removed to prevent it from being passed into the data store call
+            del(qs['rtpl_default'])
 
         store = self.stores[url.netloc]
         key = url.path[1:].strip() # path always starts with a '/'; cut it out
@@ -281,8 +294,13 @@ class Retemplate(object):
                 logging.info('Store {} got value \'{}\' for key \'{}\' and query \'{}\''.format(
                     store.name, value, key, url.query))
         except RetrievalError:
-            logging.error('Failed to resolve value for URI: {}'.format(uri))
-            raise
+            if not default:
+                logging.error('Failed to resolve value for URI: {}'.format(uri))
+                raise
+            else:
+                logging.error('Failed to resolve value for URI: {}, using default value: {}'.format(
+                    uri, default))
+                value = default
         return value
 
     def read_template(self):
@@ -410,12 +428,18 @@ class Retemplate(object):
         '''
 
         try:
-            logging.info("Writing {}; setting ownership to {}:{} and mode to {}".format(
-                self.target, self.settings['owner'], self.settings['group'], self.settings['chmod']))
+            logging.info('Writing {}'.format(self.target))
             with open(self.target, 'w') as fh:
                 fh.write(content)
-            shutil.chown(self.target, user=self.settings['owner'], group=self.settings['group'])
-            subprocess.run([ 'chmod', self.settings['chmod'], self.target ])
+            owner = self.settings['owner'] if 'owner' in self.settings else None
+            group = self.settings['group'] if 'group' in self.settings else None
+            chmod = self.settings['chmod'] if 'chmod' in self.settings else None
+            if owner or group:
+                logging.info('Setting ownership of {} to {}:{}'.format(self.target, owner, group))
+                shutil.chown(self.target, user=owner, group=group)
+            if chmod:
+                logging.info('Setting mode of {} to {}'.format(self.target, chmod))
+                subprocess.run([ 'chmod', self.settings['chmod'], self.target ])
             return True
         except IOError:
             logging.error('Cannot write target file {}'.format(self.target))
